/* @(#) $Id: config.h,v 1.15 2006/11/18 23:31:18 dcid Exp $ */

/* Copyright (C) 2003-2006 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */

 

#ifndef _CONFIG__H

#define _CONFIG__H

#include "config/config.h"
#include "config/global-config.h"

_Config Config;  /* Global Config structure */



#endif
