/* @(#) $Id: sign_log.c,v 1.1 2006/08/28 18:45:26 dcid Exp $ */

/* Copyright (C) 2006 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software 
 * Foundation
 */

#include "shared.h"

/* gzips a log file */
void OS_SignLog(char *logfile)
{
    return;
}
	  

/* EOF */
