/* @(#) $Id: alerts.h,v 1.2 2006/11/25 14:30:17 dcid Exp $ */

/* Copyright (C) 2004,2005 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software 
 * Foundation
 */

/* Global  alert header */

#ifndef _ALERT__H

#define _ALERT__H

#include "mail.h"
#include "log.h"
#include "exec.h"
#include "getloglocation.h"

#endif
