/* @(#) $Id: regex_op.h,v 1.3 2007/01/04 19:15:59 dcid Exp $ */

/* Copyright (C) 2005 Daniel B. Cid <dcid@ossec.net>
 * All rights reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


#ifndef OS_REGEX_OP_H
#define OS_REGEX_OP_H

/* Posix regex pattern matching */
int OS_PRegex(char *str, char *regex);


#endif
