/* @(#) $Id: wait_op.h,v 1.1 2006/08/29 19:35:42 dcid Exp $ */

/* Copyright (C) 2006 Daniel B. Cid <dcid@ossec.net>
 * All rights reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */

#ifndef __WAIT_OP_H
#define __WAIT_OP_H 


void os_setwait();

void os_delwait();

void os_wait();

#endif
