/* @(#) $Id: mem_op.h,v 1.2 2007/01/04 19:15:59 dcid Exp $ */

/* Copyright (C) 2004 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


#ifndef __MEM_H

#define __MEM_H

void ClearStrMem(char *ch1, char **ch2);

#endif
