/* @(#) $Id: sig_op.h,v 1.2 2007/01/04 19:15:59 dcid Exp $ */

/* Copyright (C) 2004 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


/* Functions to handle signal manipulation
 */

#ifndef __SIG_H

#define __SIG_H

void HandleSIG();
void HandleSIGPIPE();

/* Start signal manipulation */
void StartSIG(char *process_name);

#endif
