/* @(#) $Id: fts.h,v 1.4 2007/01/04 19:15:58 dcid Exp $ */

/* Copyright (C) 2004,2005 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


#ifndef __FTS_H

#define __FTS_H


/* FTS queues */
#define FTS_QUEUE "/queue/fts/fts-queue"
#define IG_QUEUE  "/queue/fts/ig-queue"

#endif
